require "core"
require "dbc_file"

# This class assumes WRATH data
# sample on extending dbcfile for re-manipulation of data
class WorldMapArea < DBCFile
  # Fied info
  # 0: ID
  # 1: map id
  # 2: area id
  # 3: Name
  # 4: y1
  # 5: y2
  # 6: x1
  # 7: x2
  # 8: virtualID
  # 9: dungone map
  # 10: Phase I think
  def initialize(path)
      super("#{path}#{File::SEPARATOR}WorldMapArea.dbc","iiisffffIIII")
  end
  def get_zone_data()
    zone_info = Hash.new
    #Fields are 0 indexed
    @records.each do |record|
      puts "MapID #{record[1]} = #{record[3]}"
      zone_info[record[3]] = {
        :uly => record[4], 
        :lry => record[5], 
        :ulx => record[6], 
        :lrx => record[7], 
        :width => (record[5].to_f - record[4].to_f).abs,
        :height => (record[7].to_f-record[6].to_f).abs,
        :mapid => record[1],
        :area_id => record[2],
        :dungeon_map => record[9],
        :virtual_id => record[0],
        :phase_id => record[10]
        }
    end
    return zone_info
  end
end
class DungeonMap < DBCFile
  #Field info
  #0 subID
  #1 worldmapID
  #2 floor
  #3 uly
  #4 lry
  #5 ulx
  #6 lrx
  #7 i dont know
  def initialize(path)
      super("#{path}#{File::SEPARATOR}DungeonMap.dbc","iiiffffi")
      @map_floors = Hash.new
      @floor_info = Hash.new
      @records.each do |record|
          if record[1] == 649 and record[0] == 94 
            next
          end
          if record[1] == 650
            next
          end
          if @map_floors.has_key?(record[1]) 
            val = @map_floors[record[1]]
            val = val +1
            @map_floors[record[1]] = val
          else
            @map_floors[record[1]] = 1
            val = 1
            @floor_info[record[1]] = Array.new
          end
          @floor_info[record[1]] << {
            :floor => record[2],
            :uly => record[3],
            :lry => record[4],
            :ulx => record[5],
            :lrx => record[6],
            :width => (record[4].to_f - record[3].to_f).abs,
            :height => (record[6].to_f-record[5].to_f).abs,
            :area_id => record[7]
          }
      end
  end
  def get_floors(mapID, areaID)
    if mapID == 571 and areaID != 4395
      return 0
    end
    if @map_floors.has_key?(mapID) and mapID > 0
      return @map_floors[mapID]
    else
      return 0
    end
  end
  def get_floor_data(mapID)
    return @floor_info[mapID]
  end
end
# Cmdline Usage
if ARGV.nil? 
  puts "Usage: #{$0} <path to dbc files>"
  exit(0)
end
path = ARGV[0]
unless File.directory?(path)
  puts "Usage: #{$0} <path to dbc files>"
  exit(0)  
end

zoneData = WorldMapArea.new("#{path}") 
floorMap = DungeonMap.new("#{path}")

zoneMaps = zoneData.get_zone_data
max_zone= 81
final_data = Array.new
zoneMaps.each_pair do |map,data|
  floors = floorMap.get_floors(data[:mapid],data[:area_id])
  floor_data =floorMap. get_floor_data(data[:mapid])
  if data[:dungeon_map] == -1
    floors = floors + 1
  end
  #puts "Map #{map} AreaID #{data[:virtual_id]}"
  line = "mapData[\"#{map}\"] = {['floors'] = #{floors},['area_id']= #{data[:virtual_id]},['phase'] = #{data[:phase_id]},"
  if floors == 0 or data[:dungeon_map] == -1
    line = line + "\n[1] = {#{data[:width]},#{data[:height]},#{data[:ulx]},#{data[:uly]},#{data[:lrx]},#{data[:lry]}},"
  end
  if floors > 0
    base_floor = 0
    if data[:dungeon_map] == -1
      base_floor = 1
    end
    floor_data.each do |floor|
      line = line + "\n[#{floor[:floor] + base_floor}] = {#{floor[:width]},#{floor[:height]},#{floor[:ulx]},#{floor[:uly]},#{floor[:lrx]},#{floor[:lry]}},"
    end
  end
  line = line + "}"
  final_data << line
end

final_data.each do |rcd|
  if rcd then
    puts rcd
  end
end